@@ -32,6 +32,9 @@ Code details
 ~~~~~~~~~~~~
 """
 
+import collections
+import numbers
+
 # pylint: disable=invalid-name
 from enum import Enum, auto
 from typing import FrozenSet, Iterable, List, Optional, Sequence, Union
@@ -46,8 +49,10 @@ from pennylane import CircuitGraph, QuantumFunctionError, QubitDevice
 from pennylane import numpy as np
 from pennylane.measurements import Expectation, Probability, Sample, State, Variance
 from pennylane.operation import Observable, Operation
+from pennylane.ops.qubit.hamiltonian import Hamiltonian
 
 from braket.pennylane_plugin.translation import (
+    get_adjoint_gradient_result_type,
     supported_operations,
     translate_operation,
     translate_result,
@@ -134,21 +139,70 @@ class BraketQubitDevice(QubitDevice):
         """QuantumTask: The task corresponding to the last run circuit."""
         return self._task
 
-    def _pl_to_braket_circuit(self, circuit, **run_kwargs):
+    def _pl_to_braket_circuit(self, circuit: CircuitGraph, compute_gradient=False, **run_kwargs):
         """Converts a PennyLane circuit to a Braket circuit"""
         braket_circuit = self.apply(
             circuit.operations,
             rotations=None,  # Diagonalizing gates are applied in Braket SDK
+            use_unique_params=compute_gradient,
             **run_kwargs,
         )
-        for observable in circuit.observables:
-            dev_wires = self.map_wires(observable.wires).tolist()
-            translated = translate_result_type(observable, dev_wires, self._braket_result_types)
-            if isinstance(translated, tuple):
-                for result_type in translated:
-                    braket_circuit.add_result_type(result_type)
-            else:
-                braket_circuit.add_result_type(translated)
+        if compute_gradient:
+            braket_circuit = self._apply_gradient_result_type(circuit, braket_circuit)
+        else:
+            for observable in circuit.observables:
+                dev_wires = self.map_wires(observable.wires).tolist()
+                translated = translate_result_type(observable, dev_wires, self._braket_result_types)
+                if isinstance(translated, tuple):
+                    for result_type in translated:
+                        braket_circuit.add_result_type(result_type)
+                else:
+                    braket_circuit.add_result_type(translated)
+        return braket_circuit
+
+    def _apply_gradient_result_type(self, circuit, braket_circuit):
+        """Adds the AdjointGradient result type to the braket_circuit with the first observable in
+        circuit.observables. This fails for circuits with multiple observables"""
+        if len(circuit.observables) != 1:
+            raise ValueError(
+                f"Braket can only compute gradients for circuits with a single expectation"
+                f" observable, not {len(circuit.observables)} observables. If your QNode "
+                f"returns the expval of a single Hamiltonian observable and you're still seeing "
+                f"this error, try recreating H = qml.Hamiltonian(H.coeffs, H.ops) before "
+                f"returning expval(H). PennyLane transforms Hamiltonian observables if a grouping"
+                f" is set, and this clears any grouping. See "
+                f"https://docs.aws.amazon.com/braket/latest/developerguide/hybrid.html for more."
+            )
+        pl_observable = circuit.observables[0]
+        if pl_observable.return_type != Expectation:
+            raise ValueError(
+                f"Braket can only compute gradients for circuits with a single expectation"
+                f" observable, not a {pl_observable.return_type} observable."
+            )
+        if isinstance(pl_observable, Hamiltonian):
+            targets = [self.map_wires(op.wires) for op in pl_observable.ops]
+        else:
+            targets = self.map_wires(pl_observable.wires).tolist()
+        param_index = 0
+        params_not_to_diff = {}
+        for operation in circuit.operations:
+            for p in operation.parameters:
+                if param_index not in circuit.trainable_params:
+                    params_not_to_diff[f"p_{param_index}"] = p
+                param_index += 1
+
+        # bind all the non-trainable parameters since they don't need to be parameters in the
+        # Amazon Braket service.
+        braket_circuit = braket_circuit(**params_not_to_diff)
+
+        braket_circuit.add_result_type(
+            get_adjoint_gradient_result_type(
+                pl_observable,
+                targets,
+                self._braket_result_types,
+                [f"p_{param_index}" for param_index in circuit.trainable_params],
+            )
+        )
         return braket_circuit
 
     def statistics(
@@ -173,7 +227,6 @@ class BraketQubitDevice(QubitDevice):
                     "Unsupported return type specified for observable {}".format(obs.name)
                 )
             results.append(self._get_statistic(braket_result, obs))
-
         return results
 
     def _braket_to_pl_result(self, braket_result, circuit):
@@ -182,6 +235,29 @@ class BraketQubitDevice(QubitDevice):
         # Compute the required statistics
         results = self.statistics(braket_result, circuit.observables)
 
+        ag_results = [
+            result
+            for result in braket_result.result_types
+            if result.type.type == "adjoint_gradient"
+        ]
+
+        if ag_results:
+            # adjoint gradient results are a "ragged nested sequences (which is a list-or-tuple of
+            # lists-or-tuples-or ndarrays with different lengths or shapes)", so we have to set
+            # dtype="object", otherwise numpy will throw a warning
+
+            # whenever the adjoint gradient result type is present, it should be the only result
+            # type, which is why this changing of dtype works. If we ever change this plugin
+            # to submit another result type alongside adjoint gradient, this logic will need to
+            # change.
+            return np.asarray(
+                [
+                    np.asarray(result, dtype="object")
+                    if isinstance(result, collections.abc.Sequence)
+                    else result
+                    for result in results
+                ]
+            )
         # Ensures that a combination with sample does not put
         # single-number results in superfluous arrays
         all_sampled = all(obs.return_type is Sample for obs in circuit.observables)
@@ -208,29 +284,47 @@ class BraketQubitDevice(QubitDevice):
         else:
             return {"braket_failed_task_id": task.id}
 
-    def execute(self, circuit: CircuitGraph, **run_kwargs) -> np.ndarray:
+    def execute(self, circuit: CircuitGraph, compute_gradient=False, **run_kwargs) -> np.ndarray:
         self.check_validity(circuit.operations, circuit.observables)
-        self._circuit = self._pl_to_braket_circuit(circuit, **run_kwargs)
-        self._task = self._run_task(self._circuit)
+        self._circuit = self._pl_to_braket_circuit(
+            circuit, compute_gradient=compute_gradient, **run_kwargs
+        )
+        param_index = 0
+        param_dict = {}
+        for operation in circuit.operations:
+            for p in operation.parameters:
+                if isinstance(p, numbers.Number):
+                    param_name = f"p_{param_index}"
+                    param_dict[param_name] = p
+                    param_index += 1
+        self._task = self._run_task(self._circuit, inputs=param_dict)
         braket_result = self._task.result()
 
         if self.tracker.active:
             tracking_data = self._tracking_data(self._task)
             self.tracker.update(executions=1, shots=self.shots, **tracking_data)
             self.tracker.record()
-
         return self._braket_to_pl_result(braket_result, circuit)
 
     def apply(
-        self, operations: Sequence[Operation], rotations: Sequence[Operation] = None, **run_kwargs
+        self,
+        operations: Sequence[Operation],
+        rotations: Sequence[Operation] = None,
+        use_unique_params=False,
+        **run_kwargs,
     ) -> Circuit:
         """Instantiate Braket Circuit object."""
         rotations = rotations or []
         circuit = Circuit()
 
         # Add operations to Braket Circuit object
+        param_index = 0
         for operation in operations + rotations:
-            gate = translate_operation(operation)
+            param_names = [f"p_{param_index+i}" for i, p in enumerate(operation.parameters)]
+            param_index += len(operation.parameters)
+            gate = translate_operation(
+                operation, use_unique_params=use_unique_params, param_names=param_names
+            )
             dev_wires = self.map_wires(operation.wires).tolist()
             ins = Instruction(gate, dev_wires)
             circuit.add_instruction(ins)
@@ -247,12 +341,11 @@ class BraketQubitDevice(QubitDevice):
         supported_result_types = self._device.properties.action[
             "braket.ir.openqasm.program"
         ].supportedResultTypes
-
         self._braket_result_types = frozenset(
             result_type.name for result_type in supported_result_types
         )
 
-    def _run_task(self, circuit):
+    def _run_task(self, circuit, inputs=None):
         raise NotImplementedError("Need to implement task runner")
 
     def _get_statistic(self, braket_result, observable):
@@ -285,7 +378,6 @@ class BraketAwsQubitDevice(BraketQubitDevice):
         aws_session (Optional[AwsSession]): An AwsSession object created to manage
             interactions with AWS services, to be supplied if extra control
             is desired. Default: None
-        parallel (bool): Indicates whether to use parallel execution for gradient calculations.
             Default: False
         max_parallel (int, optional): Maximum number of tasks to run on AWS in parallel.
             Batch creation will fail if this value is greater than the maximum allowed concurrent
@@ -348,7 +440,6 @@ class BraketAwsQubitDevice(BraketQubitDevice):
 
     @property
     def parallel(self):
-        """bool: True if gradient calculations are evaluated in parallel."""
         return self._parallel
 
     def batch_execute(self, circuits, **run_kwargs):
@@ -395,16 +486,51 @@ class BraketAwsQubitDevice(BraketQubitDevice):
             for braket_result, circuit in zip(braket_results_batch, circuits)
         ]
 
-    def _run_task(self, circuit):
+    def _run_task(self, circuit, inputs=None):
         return self._device.run(
             circuit,
             s3_destination_folder=self._s3_folder,
             shots=0 if self.analytic else self.shots,
             poll_timeout_seconds=self._poll_timeout_seconds,
             poll_interval_seconds=self._poll_interval_seconds,
+            inputs=inputs or {},
             **self._run_kwargs,
         )
 
+    def capabilities(self=None):
+        """Add support for AG on sv1"""
+        # normally, we'd just call super().capabilities() here, but super()
+        # resolution doesn't work when you override a classmethod with an instance method
+        capabilities = BraketQubitDevice.capabilities().copy()
+        # if this method is called as a class method, don't add provides_jacobian since
+        # we don't know if the device is sv1
+        if self and "AdjointGradient" in self._braket_result_types:
+            capabilities.update(provides_jacobian=True)
+        return capabilities
+
+    def execute_and_gradients(self, circuits, **kwargs):
+        """Execute a list of circuits and calculate their gradients.
+        Returns a list of circuit results and a list of gradients/jacobians, one of each
+        for each circuit in circuits.
+        of floats, 1 float for every instance of a trainable parameter in a gate in the circuit.
+        Functions like qml.grad or qml.jacobian then use that format to generate a per-parameter
+        format.
+        """
+        res = []
+        jacs = []
+        for circuit in circuits:
+            if not circuit.trainable_params:
+                new_res = self.execute(circuit, compute_gradient=False)
+                # don't bother computing a gradient when there aren't any trainable parameters.
+                new_jac = np.tensor([])
+            else:
+                results = self.execute(circuit, compute_gradient=True)
+                new_res, new_jac = results[0]
+            res.append(new_res)
+            jacs.append(new_jac)
+
+        return res, jacs
+
 
 class BraketLocalQubitDevice(BraketQubitDevice):
     r"""Amazon Braket LocalSimulator qubit device for PennyLane.
@@ -436,7 +562,7 @@ class BraketLocalQubitDevice(BraketQubitDevice):
         device = LocalSimulator(backend)
         super().__init__(wires, device, shots=shots, **run_kwargs)
 
-    def _run_task(self, circuit):
+    def _run_task(self, circuit, inputs=None):
         return self._device.run(
             circuit, shots=0 if self.analytic else self.shots, **self._run_kwargs
         )
